import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib.dates import DateFormatter, HourLocator
from .utils import get_gaps


def plot(temp_data, parameter_name, yaxis_height_limit=10):
    """
    Creates a contour plot of the given parameter for the Microwave Radiometer data

    Args:
    temp_data (DataFrame): the Microwave Radiometer data to be plotted
    parameter_name (str): the name of the parameter to be plotted
    yaxis_height_limit (int): the maximum value of the y-axis

    Returns:
    fig (Figure): the figure object containing the plot
    """

    # Define color range for plotting based on scan angle
    if temp_data['400'].unique()[0] == '401':
        clevs = np.linspace(270, 300, 16, endpoint=True)
    elif temp_data['400'].unique()[0] == '404':
        clevs = np.linspace(40, 100, 7, endpoint=True)
    elif temp_data['400'].unique()[0] == '403':
        clevs = np.linspace(0, .01, endpoint=True)
    elif temp_data['400'].unique()[0] == '402':
        clevs = np.linspace(0, 15, endpoint=True)

    # Remove gaps in data
    temp_data = get_gaps(temp_data)

    # Create figure and plot
    fig = plt.figure(figsize=[25, 5], dpi=200)
    cs_t = plt.contourf(temp_data.index, temp_data.columns, temp_data.astype(float).T, clevs, cmap='turbo', extend='both')
    cs1 = plt.colorbar(cs_t, pad=0.008)
    cs1.ax.tick_params(labelsize=16, length=2)
    plt.xlabel("Datetime (UTC)", fontsize=20)
    plt.ylabel("Height (Km)", fontsize=20)
    plt.title("Microwave Radiometer" + '  ' * 6 + parameter_name + ' ' * 2, fontsize=22)
    plt.gca().xaxis.set_major_locator(mdates.DayLocator(interval=1))
    plt.gca().xaxis.set_minor_locator(HourLocator(np.arange(0, 25, 3)))  # enter hours difference
    plt.gca().xaxis.set_major_formatter(mdates.DateFormatter('%H\n%d%b'))
    plt.gca().xaxis.set_minor_formatter(DateFormatter('%H'))
    plt.tick_params(axis='both', which='major', labelsize=15, direction='out', length=8, width=2)
    plt.tick_params(axis='both', which='minor', labelsize=15, direction='out', length=8, width=2)
    plt.ylim(0, yaxis_height_limit)

    return fig



"""
pymwr - Python Package to interact with MWR data
==================================
Top-level package (:mod:`pymwr`)
==================================
.. currentmodule:: pymwr
"""


from .read import read
from .utils import get_dates, get_gaps, get_height_levels
from .utils import get_site_coords, get_scan_data
from .plot import plot


__all__ = [s for s in dir() if not s.startswith('_')]



import pandas as pd
import datetime
import numpy as np


def get_dates(mwr_data: pd.DataFrame) -> dict:
    """
    Extracts the starting dates of each file in a Level 2 MWR dataset.

    Parameters:
    -----------
    mwr_data: pd.DataFrame
        A Pandas DataFrame containing the Level 2 MWR data.

    Returns:
    --------
    dict
        A dictionary containing a list of starting dates for each file in the dataset.
    """
    data_records = mwr_data[mwr_data.eq('Record').any(axis=1)]
    col_400 = data_records[data_records.eq('400').any(axis=1)].drop_duplicates().values[0]
    data_dates = mwr_data.loc[data_records[data_records.eq('400').any(axis=1)].index+1]
    data_dates.columns = col_400
    return {'Each file starting date': list(data_dates['Date/Time'].values)}


def get_site_coords(mwr_data):
    """
    Extracts the latitude and longitude from microwave radiometer data.

    Parameters:
    -----------
    mwr_data : pandas DataFrame
        A pandas DataFrame containing microwave radiometer data.

    Returns:
    --------
    dict
        A dictionary containing the extracted latitude and longitude.

    Raises:
    -------
    None
    """
    data_records = mwr_data[mwr_data.eq('Record').any(axis=1)]
    col_pos = list(data_records[data_records.eq('Latitude').any(axis=1)].values[0])
    temp_df = mwr_data[mwr_data.eq('31').any(axis=1)]
    temp_df.columns = col_pos
    latitude, longitude = temp_df[['Latitude', 'Longitude']].drop_duplicates().astype(float).values[0]
    return {'Latitude': latitude, 'Longitude': longitude}


def get_height_levels(mwr_data: pd.DataFrame) -> dict:
    """
    Extracts the height levels of the MWR measurements in a Level 2 MWR dataset.

    Parameters:
    -----------
    mwr_data: pd.DataFrame
        A Pandas DataFrame containing the Level 2 MWR data.

    Returns:
    --------
    dict
        A dictionary containing a list of height levels (in km) for the MWR measurements.
    """
    data_records = mwr_data[mwr_data.eq('Record').any(axis=1)]
    temp = list(data_records[data_records.eq('LV2 Processor').any(axis=1)].values[0])
    mwr_ht = []
    for i in temp:
        try:
            mwr_ht.append(float(i))
        except ValueError:
            pass
    mwr_ht = mwr_ht[mwr_ht.index(0):]
    return {'MWR height levels (km)': mwr_ht}


def get_scan_data(mwr_data: pd.DataFrame, scan_angle: int, scan_code: int, quality_flag: str = '1') -> pd.DataFrame:
    """
    Extracts MWR profile data for a specific scan angle and scan code from a Level 2 MWR dataset.

    Parameters:
    -----------
    mwr_data: pd.DataFrame
        A Pandas DataFrame containing the Level 2 MWR data.
    scan_angle: int
        The scan angle to extract data for.
    scan_code: int
        The scan code to extract data for.
    quality_flag: str (default '1')
        The data quality flag to use for filtering.

    Returns:
    --------
    pd.DataFrame
        A Pandas DataFrame containing the MWR profile data for the specified scan angle and scan code.
    """
    data_records = mwr_data[mwr_data.eq('Record').any(axis=1)]
    col_lv2processor = list(data_records[data_records.eq('LV2 Processor').any(axis=1)].values[0])
    temp_df = mwr_data.copy()
    temp_df.columns = col_lv2processor
    profile_data = temp_df[(temp_df['400'] == str(scan_angle)) & (temp_df['LV2 Processor'] == str(scan_code))]
    profile_data['Date'] = pd.to_datetime(profile_data['Date/Time'])
    profile_data = profile_data[profile_data['DataQuality'] == quality_flag]
    profile_data = profile_data.drop(['Date/Time'], axis=1)
    return profile_data


def get_gaps(df):
    """Identify data gaps greater than or equal to 15 minutes
    df: pandas.DataFrame - DataFrame containing the data

    Returns
    -------
    pandas.DataFrame - DataFrame containing the original data plus any added rows to fill the gaps.
    """
    mwr1 = df.dropna(axis=1).copy()
    x1 = []
    mwr1.reset_index(drop=True, inplace=True)
    for kk in range(len(mwr1)-1):
        if (mwr1['Date'][kk+1]-mwr1['Date'][kk]) >= datetime.timedelta(minutes=15):
            x1.append(mwr1['Date'][kk]+datetime.timedelta(seconds=1))
            x1.append(mwr1['Date'][kk+1]-datetime.timedelta(seconds=1))

    x2 = pd.DataFrame({'Date': x1})
    if len(x2) > 0:
        df2 = pd.concat([mwr1, x2])
        df2 = df2.sort_values(by='Date', ascending=True)
    else:
        df2 = mwr1.sort_values(by='Date', ascending=True)
    df2 = df2.set_index('Date')
    df2 = df2.drop(['Record', '400', 'LV2 Processor', 'DataQuality'], axis=1)
    df2 = df2.replace('', np.nan).dropna(axis=1)
    df2.columns = [float(i) for i in df2.columns]
    return df2



import pandas as pd
import csv
import glob


def read(data_dir: str) -> pd.DataFrame:
    """
    Read Level 2 MWR data from CSV files in a directory.

    Parameters:
    -----------
    data_dir: str
        Path to the directory containing MWR data CSV files.

    Returns:
    --------
    pd.DataFrame
        A Pandas DataFrame containing the MWR data from all CSV files in the directory.
    """
    mwr_files = glob.glob(data_dir + '/*.csv')
    mwr_data = []
    for filename in mwr_files:
        with open(filename, 'r') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')
            for row in reader:
                mwr_data.append(row)
    mwr_data = pd.DataFrame(mwr_data)
    return mwr_data
